import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/painting.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:intl/intl.dart';
import 'package:puma_pay/extensions/extProduct.dart';
import 'package:puma_pay/extensions/extPurchaseDetail.dart';
import 'package:puma_pay/redux/pumaPayActions.dart';
import 'package:puma_pay/redux/pumaPayState.dart';
import 'package:redux/redux.dart';

class PumaPayDebugContainer<T extends VirtualPumaPayState>
    extends StatelessWidget {
  PumaPayDebugContainer();
  final String title = "PumaPayDebug";
  @override
  Widget build(BuildContext context) {
    return StoreConnector<T, _ViewModel>(
      converter: (Store<T> store) => _ViewModel.create(store),
      builder: (BuildContext context, _ViewModel viewModel) =>
          _createBody(context, viewModel),
    );
  }

  Widget _createBody(BuildContext context, _ViewModel vm) {
    var format = new DateFormat("yMd");
    return Builder(builder: (context) {
      return Column(
        children: [
          Container(
            width: MediaQuery.of(context).size.width,
            color: Colors.blueGrey.withAlpha(20),
            child: Padding(
              padding: const EdgeInsets.all(8.0),
              child: Wrap(
                alignment: WrapAlignment.center,
                spacing: 4.0,
                children: [
                  Column(
                    children: [
                      Icon(vm.pumaPayState.storeAvailable
                          ? Icons.check
                          : Icons.motion_photos_off),
                      Text(
                        "Store",
                        style: Theme.of(context).textTheme.caption,
                      ),
                    ],
                  ),
                  Column(
                    children: [
                      if (vm.pumaPayState.productsLoading)
                        Container(
                            width: 20,
                            height: 20,
                            child: CircularProgressIndicator()),
                      if (!vm.pumaPayState.productsLoading)
                        Icon(Icons.motion_photos_off),
                      Text(
                        "Products loading",
                        style: Theme.of(context).textTheme.caption,
                      ),
                    ],
                  ),
                  Column(
                    children: [
                      if (vm.pumaPayState.purchasePending)
                        Container(
                            width: 20,
                            height: 20,
                            child: CircularProgressIndicator()),
                      if (!vm.pumaPayState.purchasePending)
                        Icon(Icons.motion_photos_off),
                      Text(
                        "Purchase Pending",
                        style: Theme.of(context).textTheme.caption,
                      ),
                    ],
                  ),
                  Column(
                    children: [
                      if (vm.pumaPayState.restoringPurchase)
                        Container(
                            width: 20,
                            height: 20,
                            child: CircularProgressIndicator()),
                      if (!vm.pumaPayState.restoringPurchase)
                        Icon(Icons.motion_photos_off),
                      Text(
                        "Restoring Purchase",
                        style: Theme.of(context).textTheme.caption,
                      ),
                    ],
                  ),
                ],
              ),
            ),
          ),
          Expanded(
            child: ListView(
              padding: const EdgeInsets.only(
                  left: 16.0, right: 16.0, bottom: 64.0, top: 16),
              children: [
                Wrap(
                  spacing: 16,
                  runSpacing: 16,
                  children: [
                    Column(
                      children: [
                        Text(
                          "Configuration",
                          style: Theme.of(context).textTheme.headline4,
                        ),
                        ListTile(
                          leading: Text("productIDs"),
                          trailing: Text(vm.pumaPayState.productIDs.toString()),
                        ),
                        ListTile(
                          leading: Text("appleAppStorePassword"),
                          trailing: Text(
                              "**** ${vm.pumaPayState.appleAppStorePassword.toString().substring(vm.pumaPayState.appleAppStorePassword!.length - 5)}"),
                        ),
                        ListTile(
                          leading: Text("appStoreSandbox"),
                          trailing:
                              Text(vm.pumaPayState.appStoreSandbox.toString()),
                        ),
                        ListTile(
                          leading: Text("verifyEndpoint"),
                          trailing:
                              Text(vm.pumaPayState.verifyEndpoint.toString()),
                        ),
                      ],
                    ),
                    Divider(),
                    Column(
                      children: [
                        Text(
                          "Products Available",
                          style: Theme.of(context).textTheme.headline4,
                        ),
                        for (var product in vm.pumaPayState.products)
                          Column(
                            children: [
                              ListTile(
                                  title: Text("${product.title}"),
                                  leading: Icon(Icons.local_mall),
                                  subtitle: Column(
                                    mainAxisAlignment: MainAxisAlignment.start,
                                    crossAxisAlignment:
                                        CrossAxisAlignment.start,
                                    children: [
                                      Text("${product.description}"),
                                      Text(
                                        "ID: ${product.id}",
                                        style:
                                            Theme.of(context).textTheme.caption,
                                      ),
                                      Text(
                                        "${product.price}",
                                        style:
                                            Theme.of(context).textTheme.caption,
                                      ),
                                      Text(
                                        product.isValidString(),
                                        style: Theme.of(context)
                                            .textTheme
                                            .bodyText2!
                                            .merge(TextStyle(
                                                fontWeight: FontWeight.bold,
                                                color: product.validColor())),
                                      ),
                                    ],
                                  ),
                                  trailing: GestureDetector(
                                    onTap: () {
                                      vm.store.dispatch(PurchaseProductAction(
                                          productID: product.id));
                                    },
                                    child: Chip(
                                        label: Text(
                                      "Buy Now",
                                      style:
                                          Theme.of(context).textTheme.caption,
                                    )),
                                  )),
                              if (product.purchaseDetails.length > 0)
                                Column(
                                  mainAxisAlignment: MainAxisAlignment.start,
                                  crossAxisAlignment: CrossAxisAlignment.start,
                                  children: [
                                    Row(
                                      children: [
                                        Expanded(
                                          child: Card(
                                            child: Padding(
                                              padding:
                                                  const EdgeInsets.all(8.0),
                                              child: Text(
                                                "Puchased ${product.purchaseDetails.length} times:",
                                                style: TextStyle(
                                                    fontWeight:
                                                        FontWeight.bold),
                                              ),
                                            ),
                                          ),
                                        ),
                                      ],
                                    ),
                                    for (var purchaseDetail
                                        in product.purchaseDetails)
                                      ListTile(
                                        leading: Column(
                                          mainAxisAlignment:
                                              MainAxisAlignment.start,
                                          crossAxisAlignment:
                                              CrossAxisAlignment.start,
                                          children: [
                                            Text("${purchaseDetail.productID}"),
                                            Text(
                                              "Purchase ID: ${purchaseDetail.purchaseID}",
                                              style: Theme.of(context)
                                                  .textTheme
                                                  .caption,
                                            ),
                                          ],
                                        ),
                                        trailing: Column(
                                          crossAxisAlignment:
                                              CrossAxisAlignment.end,
                                          children: [
                                            if (purchaseDetail
                                                    .transactionDate !=
                                                null)
                                              Text(
                                                  "${format.format(DateTime.fromMillisecondsSinceEpoch(int.parse(purchaseDetail.transactionDate!)))}"),
                                            Text(
                                              purchaseDetail.isValidString(),
                                              style: Theme.of(context)
                                                  .textTheme
                                                  .caption!
                                                  .merge(TextStyle(
                                                      fontWeight:
                                                          FontWeight.bold,
                                                      color: purchaseDetail
                                                          .validColor())),
                                            ),
                                          ],
                                        ),
                                      ),
                                  ],
                                )
                            ],
                          ),
                      ],
                    ),
                    Divider(),
                  ],
                ),
              ],
            ),
          ),
          Container(
            width: MediaQuery.of(context).size.width,
            color: Colors.blue,
            child: SafeArea(
                child: Wrap(
              alignment: WrapAlignment.center,
              spacing: 16.0,
              children: [
                GestureDetector(
                  onTap: () => vm.store.dispatch(IsStoreAvailableAction()),
                  child: Chip(
                    backgroundColor: Colors.white,
                    label: Text("Check Availability",
                        style: Theme.of(context)
                            .textTheme
                            .caption!
                            .merge(TextStyle(color: Colors.blue))),
                  ),
                ),
                GestureDetector(
                  onTap: () => vm.store.dispatch(GetProductsAction(
                      productIDs:
                          vm.store.state.pumaPayState.productIDs.toList())),
                  child: Chip(
                    backgroundColor: Colors.white,
                    label: Text("Get Products",
                        style: Theme.of(context)
                            .textTheme
                            .caption!
                            .merge(TextStyle(color: Colors.blue))),
                  ),
                ),
                GestureDetector(
                  onTap: () => vm.store.dispatch(RestorePurchaseAction()),
                  child: Chip(
                    backgroundColor: Colors.white,
                    label: Text("Restore",
                        style: Theme.of(context)
                            .textTheme
                            .caption!
                            .merge(TextStyle(color: Colors.blue))),
                  ),
                ),
              ],
            )),
          )
        ],
      );
    });
  }
}

class _ViewModel {
  PumaPayState pumaPayState;
  Store store;
  _ViewModel(this.pumaPayState, this.store);

  factory _ViewModel.create(Store<VirtualPumaPayState> store) {
    return _ViewModel(store.state.pumaPayState, store);
  }
}
